from Globals import DTMLFile, MessageDialog, Persistent, InitializeClass
import OFS
import Acquisition
import AccessControl
from Products import ZCatalog
import base
import issue
import time
import session_manager
import session_folder

manage_add_issue_manager_form = DTMLFile('issue_manager_add', globals()) 

javascript = """
function show(id) {
	document.getElementById(id).style.visibility = 'visible';
	return true;
}
    
function hide(id) {
	document.getElementById(id).style.visibility = 'hidden';
	return true;
}
"""

css = """
body {
	background-color: #fff;
	color: #000000;
	font-family: verdana, helvetica, arial, sans-serif;
	font-size: 12px;
}

a {
	text-decoration: none;
	color: #436976;
	background-color: transparent;
}

.green { color: #090; }
.green:hover { color: #6c6; }

.red { color: #900; }
.red:hover { color: #c66; }

.yellow { color: #c90; }
.yellow:hover { color: #fc0; }

a:hover {
	color: #769ca9;
	background-color: transparent;
}

table {
	border: 0px;
}

#main {
	width: 65%;
	float: left;
	padding: 3px;
}

#wide_main {
	width: 95%;
	float: left;
	border: none 1px #ccc;
	padding: 3px;
}

#remote {
	float: right;
	padding: 5px;
	width: 30%;
	position: relative;
}


#breadcrumbs {
        padding: 5px;
        width: 65%;
}

div.hidden {
	background-color: #cff;
	position: absolute;
	visibility: hidden;
	border: 1px #999 solid;
	padding: 0.5em;
	z-index: 2;
}

input {
	border: 1px dotted #999;
	background-color: #fff;
	font-family: verdana, helvetica, arial, sans-serif;
	font-size: 10px;
	font-weight: bolder;
	text-indent: 0.3em;
	cursor : pointer;
}

.discouraged {
	color: #999;
}

pre {
	font-family: verdana, helvetica, arial, sans-serif;
}

textarea {
	border: 1px dotted #999;
	font-family: verdana, helvetica, arial, sans-serif;
	font-weight: bolder;
        font-size: 0.8em;
}

th {
	text-align: left;
       	vertical-align: top;
}

td {
	text-align: left;
       	vertical-align: top;
}

#copyright {
	color: #666;
	padding: 10px;
	padding-top: 30px;
}

h1, h2, h3, h4, h5, h6 {
	font-weight: bolder;
}

h1 {
	font-size: 150%;
}

h2 {
	font-size: 140%;
}

h3 {
	font-size: 130%;
}

h4 {
	font-size: 110%;
}

h5 {
	font-size: 80%;
}

h6 {
	font-size: 60%;
}

/* reSTX styles */

div.document blockquote {
	padding: 2px;
	margin: 5px;
}

div.document ol {
}

div.document ul li p.first {
	padding: 0px;
	margin: 0px;
}

div.document ol {
}

div.document blockquote p {
	padding-top: 2px;
	padding-bottom: 2px;
	margin-top: 2px;
	margin-bottom: 2px;
}

/* Session index listing table */

table.session {
	width: 95%;
	border: 1px;
}

table.session tr td {
	padding: 0px;
	margin: 0px;
}
"""

def manage_add_issue_manager(self, id, title='', REQUEST=None):
    """Add an Issue Manager."""
    self._setObject(id, issue_manager(id, title))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class issue_manager(
    OFS.Folder.Folder,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager,
    base.base,
    session_manager.session_manager,
    ): 
    """Issue Manager class. 

    An issueManager contains Issues.
    """

    meta_type = 'Issue Manager'
    size = 500
    deleted = 0

    manage_options = (OFS.Folder.Folder.manage_options[0],) + \
        (
            {'label': 'View',       'action': ''},
            {'label': 'Security',   'action': 'manage_access'},
        ) 

    # Needed for cutting and pasting objects.
    all_meta_types = [
        {'visibility': 'Global',
         'interfaces': [],
         'action': 'manage_add_issue',
         'permission': 'Add Issues',
         'name': 'Issue',
         'product': 'Issue Manager',
         'instance': ''},
        {'visibility': 'Global',
         'interfaces': [],
         'action': '',
         'permission': 'Add Issues',
         'name': 'Session Folder',
         'product': 'Issue Manager',
         'instance': ''}        
    ]

    security = AccessControl.ClassSecurityInfo()

    security.declareProtected('Use Issue Manager', 'index_html')
    index_html = DTMLFile('product_index', globals())
    security.declareProtected('Use Issue Manager', 'manage_add_issue')
    manage_add_issue = issue.manage_add_issue
    security.declareProtected('Use Issue Manager', 'manage_add_issue_edit')
    manage_add_issue_edit = issue.manage_add_issue_edit

    security.declarePublic('style')
    def style(self):
        """Returns stylesheet data."""
        self.REQUEST.RESPONSE.setHeader('Content-Type', 'text/css')
        return css

    security.declarePublic('javascript')
    def javascript(self):
        """Returns javascript statements."""
        return javascript

    security.declarePrivate('get_issue_manager')
    def get_issue_manager(self):
        """Returns self."""
        return self

    security.declarePrivate('get_unique_id')
    def get_unique_id(self):
        """Returns a unique identifier."""
        # This is good enough for now
        integer, float = str(time.time()).split('.')
        return integer + 'X' + float

    indexes = (
      ('type', 'FieldIndex'),
      ('state', 'FieldIndex'),
      ('get_parent_url', 'FieldIndex'),
      ('contents', 'TextIndex'),
      ('get_all_text', 'TextIndex'),
      ('get_title', 'FieldIndex'),
      ('creator', 'FieldIndex'),
      ('owner', 'FieldIndex'),
      ('deleted', 'FieldIndex'),
      ('importance', 'FieldIndex'),
      ('get_state_importance_title_sort', 'FieldIndex'),
      ('get_relative_state', 'FieldIndex'),
      ('get_relative_importance', 'FieldIndex'),
      ('users', 'KeywordIndex')
    )

    columns = ('contents', 'absolute_url', 'state',
               'type', 'get_title', 'get_number_of_issues',
               'creator', 'owner', 'deleted', 'importance')

    def __init__(self, id, title=''):
        self.id = id
        self.title = title
        ZCatalog.ZCatalog.manage_addZCatalog(self, 'Catalog', 'Issue catalog')
        self.add_indexes()
        self.add_columns()
        self._add_session_folder()

    security.declarePrivate('add_indexes')
    def add_indexes(self):
        """Adds the indexes we're using."""
        catalog = self.Catalog
        for index in self.indexes:
            if not catalog._catalog.indexes.has_key(index[0]):
	        catalog.manage_addIndex(index[0], index[1])

    security.declarePrivate('add_columns')
    def add_columns(self):
        """Adds the columns we're using."""
        catalog = self.Catalog
        for column in self.columns:
            if not catalog._catalog.schema.has_key(column):
                self.Catalog.manage_addColumn(column)

    security.declarePrivate('get_issue_states')
    def get_issue_states(self):
        """Returns a tuple of Issue states."""
        return ('open', 'closed', 'discarded', 'suspended')

    security.declarePrivate('get_issue_types')
    def get_issue_types(self):
        """Returns a tuple of Issue types."""
        return ('alternative', 'goal', 'idea', 'info',
                'problem', 'question', 'solution', 'undefined')

    security.declarePrivate('get_importance_levels')
    def get_importance_levels(self):
        """Returns a sequence of importance levels."""
        return ((0, 'low'), (1, 'normal'), (2, 'high'))

    security.declareProtected('Manage Issue Manager', 'update_catalog')
    def update_catalog(self, start=0, sleep=0, batch_size=50, update=0):
        """Updates the catalog, reindexing all objects.

        If update is specified, it will be run for each object."""
        catalog = self.Catalog._catalog
        start = int(start)
        batch_size = int(batch_size)
        stop = 0
        if start == 0:
            for session in self.sessions.objectValues():
                session.update()
            # Remove the session adding later, FIXME
            self._add_session_folder()
            self._uids = list(catalog.uids.keys())
            self.add_indexes()
            self.add_columns()
            if self.sessions.meta_type != 'Session Folder':
                self._upgrade_session_folder()
        batch = self._uids[start:start+batch_size]
        if not batch:
            stop = 1
            del self._uids
        for uid in batch:
            try:
                object = self.unrestrictedTraverse(uid)
                if int(update):
                    object.update()
                object.index_object()
            except KeyError:
                # It doesn't exist, remove it from the catalog.
                catalog.uncatalog_object(uid)
            except AttributeError:
                # It doesn't inherit from CatalogAwareness.CatalogAware;
                # remove it from the catalog.
                catalog.uncatalog_object(uid)                
        if stop:
            self.REQUEST.RESPONSE.redirect(self.absolute_url())
        else:
            self.REQUEST.RESPONSE.redirect(self.absolute_url() + \
                                           '/update_catalog?start=' + \
                                           str(start + batch_size) + \
                                           '&update=' + str(update))

    security.declarePrivate('get_object')
    def get_object(self, id):
        """Returns the object specified by id."""
        return self.catalog_search(id=id)[0]

    def _upgrade_session_folder(self):
        """Called to upgrade the session folder."""
        session_folder.manage_addSessionFolder(self, id='sessions2')
        self.sessions2.manage_pasteObjects(self.sessions.manage_cutObjects(
            ids=self.sessions.objectIds()))
        self.manage_delObjects(ids=['sessions'])
        self.manage_renameObjects(ids=['sessions2'], new_ids=['sessions'])
        return "Success"

    security.declareProtected('Use Issue Manager', 'upgrade_session_folder')
    upgrade_session_folder = _upgrade_session_folder

    def _add_session_folder(self):
        if hasattr(self, 'aq_base'):
            if hasattr(self.aq_base, 'sessions'): return
        session_folder.manage_addSessionFolder(self, id='sessions')

InitializeClass(issue_manager)
